﻿using Microsoft.Extensions.Logging;
using TypographyBusinessLogic.BusinessLogics;
using TypographyContracts.BindingModels;
using TypographyContracts.BusinessLogicsContracts;
using TypographyContracts.SearchModels;
using TypographyContracts.StoragesContracts;
using TypographyContracts.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TypographyBusinessLogic.BusinessLogics
{
    public class PrintedLogic : IPrintedLogic
    {
        private readonly ILogger _logger;
        private readonly IPrintedStorage _snackStorage;
        public PrintedLogic(ILogger<PrintedLogic> logger, IPrintedStorage snackStorage)
        {
            _logger = logger;
            _snackStorage = snackStorage;
        }
        public List<PrintedViewModel>? ReadList(PrintedSearchModel? model)
        {
            _logger.LogInformation("ReadList. PrintedName:{PrintedName}.Id:{ Id}", model?.PrintedName, model?.Id);
            var list = model == null ? _snackStorage.GetFullList() : _snackStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public PrintedViewModel? ReadElement(PrintedSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. PrintedName:{PrintedName}.Id:{ Id}", model.PrintedName, model.Id);
            var element = _snackStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(PrintedBindingModel model)
        {
            CheckModel(model);
            if (_snackStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(PrintedBindingModel model)
        {
            CheckModel(model);
            if (_snackStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(PrintedBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_snackStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(PrintedBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.PrintedName))
            {
                throw new ArgumentNullException("Нет названия закуски",
               nameof(model.PrintedName));
            }
            if (model.Price <= 0)
            {
                throw new ArgumentNullException("Цена закуски должна быть больше 0", nameof(model.Price));
            }
            _logger.LogInformation("Printed. PrintedName:{PrintedName}.Price:{ Price}. Id: { Id}", model.PrintedName, model.Price, model.Id);
            var element = _snackStorage.GetElement(new PrintedSearchModel
            {
                PrintedName = model.PrintedName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Закуска с таким названием уже есть");
            }
        }
    }
}
